function    [kH,Ce]                                              =Permutation_alignment(xw,m,n,estH,numfreq,Rx,navg,perm_method )
%--------------------------------------------------------------------------
    P                                                            =size(Rx,3);
if navg>1
[kH,Permutation,Ce]                                             =initialperm(xw,m,n,estH,numfreq,Rx,P,perm_method );
end
end
%===============================Subfunctions===============================
function   [kH,Permutation,Ce]                                 =initialperm(xw,m,n,estH,numfreq,Rx,P,perm_method)
%--------------------------------------------------------------------------
%Q                          =(1/m)*ones(n, m);                      % matrix useful to remove scaling ambiguity
%--------------------------------------------------------------------------
%Ce                         =zeros(n^2,P,numfreq/2+1);
% Q                          =(1/m)*ones(n, m); 
for f                      =1:numfreq/2+1    
        hf                 =(squeeze(estH(:,:,f))); 
       Hf(:,:,f)           =hf;
    Wf(:,:,f)              =pinv(hf);
    newHf(:,:,f)          =hf*diag(1./sqrt(sum(abs(hf).^2)));
    psuW                   =(kat_rao(conj(newHf(:, :,f)),newHf(:, :, f)));
    Ce(:, :, f)            =reshape(Rx(:,:,:,f),m*m,P).'/(psuW).';   
   Ce(:,:,f)               =Ce(:,:,f)* diag(1./sqrt(sum(abs(Ce(:,:,f)).^2)));   
end
%---------------------------------------------------------------------------
perm_deflation          =0;
[Wf,pHf,Permutation]    =Solveperm_power(xw,Ce,Wf,Hf,perm_method,perm_deflation,n);
kH                      =cat(3, pHf, conj(pHf(:, :, end-1:-1:2)));
%--------------------------------------------------------------------------
end

function C = kat_rao(A,B)
%-------------------------------------------------------------------
% Calculate the Kathri Rao product of 2 matrices
%-------------------------------------------------------------------
        I=size(A,1);R1=size(A,2);
        J=size(B,1);R2=size(B,2);
        if R1~=R2
            error('Input matrices must have the same number of columns for Khatri Rao product')
        end
        
        C=zeros(I*J,R1);
        for j=1:R1
            C(:,j)=reshape(B(:,j)*A(:,j).',I*J,1);
        end
end  

function [Wf,Hf,Permutation]=Solveperm_power(xw,Ce,Wf,Hf,method,deflation,n)
%--------------------------------------------------------------------------
% This function is a permutation solver for frequency domain BSS problems
%--------------------------------------------------------------------------
% Parameters:
% I: number of sources
% J: number of microphones
% N: number of temporal samples of the whole signal
% T: number of frequency bins
% sT: =T/2+1 number of frequency bins really needed (symmetric spectrum, because x is real)
%---------------------------------------------------------------------------------------------------------------------
% INPUTS:
% - Ce (PxIxsT)  : Estimated power profiles of the I sources for the sT frequencies and P time blocks
% - Wf (IxJxsT)  : Estimated demixing matrices for the sT frequencies
% - Hf (JxIxsT)  : Estimated mixing matrices for the sT frequencies
% - method : string to choose the way clustering is done:
%           * if method=='kmeans', then the centroids are given by kmeans
%           algorithm and the profiles are then permuted so as to match the
%           centroid matrix as much as possible (in the sense of frobenius norm)
%           * if method=='iterative', then both centroids and permutations
%           are updated within a loop. Each centroid is updated as the
%           mean of the associated updated profiles.
% - deflation : = 0 or 1
%           * if 0, then for each frequency we try all possible
%           permutations (i.e. I!) and then select the one that leads to
%           the best centroid matrix matching
%           * if 1, then for each frequency, we associate centroid and
%           profiles by a deflation step, i.e., when a centroid has been
%           chosen already, it is removed from the list of candidates.
%-----------------------------------------------------------------------------------------------------------------
% OUTPUTS:
% - Hf of size JxIxsT  is the rightly permuted estimated mixing filter,
% accross the HALF spectrum (the missing half spectrum has to be built by symmetry outside this function)
% - Wf of size IxJxsT:  idem as Hf, but with the demixing filter
%------------------------------------------------------------------------------------------------------------------

%------- Size of the problem ------------------
I=size(Wf,1);    % number of sources
J=size(Wf,2);    % number of microphones
sT=size(Wf,3);   % = T/2 +1


%T=2*(sT-1);      % total number of frequencies
P=size(Ce,1);                  % number of frames
%------- Compute the log-power profiles
Ce=(abs(Ce));         % log power profiles log
%Ce=log((Ce)); 
%%%%%%Center the profiles by substracting the time-average
Ce=permute(Ce,[1 3 2]);
avg=mean(Ce);
Ce = Ce - avg(ones(P,1),:,:);
Ce=permute(Ce,[1 3 2]);  %%% P*M*sT
nb_perm=perms(1:I);         % holds all possible permutation of I elements
Ckmeans = zeros(I*sT, P);    % Holds the profiles to be clustered 
 for f= 1:sT
       Ckmeans((f-1)*I+1:f*I, :)       =Ce(:, :, f).'; 
 end  
%--------------------------------------------------------------------------    
if strcmp(method,'kmeans')==1
   nb_try              =5;         % number of starting points   
 [IDX,Cent]           =kmeans(Ckmeans,I,'replicates',nb_try);  
 Ckmeans              =permute(Ckmeans,[2,1]);
 Cent                 =Cent.'; 
elseif strcmp(method,'Densityclusteringmethod')==1
  [estC]              =Proposed_Permutation_clustering_method(1,Ckmeans',n);
  Cent                =estC(:,1:n);   
end
     for f=1:sT   
                 for np=1:size(nb_perm,1)               % for all permutations
                     Ce_perm=Ce(:,nb_perm(np,:),f);     % permute the profile
                     cost(np) = norm(Cent - Ce_perm,'fro');  % compute distance to centroid matrix
                 end
                 id=find(cost==min(cost));    % localize the best permutation
                 permf=nb_perm(id(1),:);      % vector of permutation indexes
                 Wf(:,:,f)=Wf(permf,:,f);     % demixing matrix after perm correction
                 Hf(:,:,f)=Hf(:,permf,f);     % mixing matrix after perm correction
                 Permutation(:,:,f)=permf;
     end    
%--------------------------------------------------------------------------- 
%             dT                       =50;
%        origfeatures              =Ckmeans.'; 
%        features              =origfeatures(:,1:I*dT);     
%         [features]            =normalizationmixutres(1,P,features);     
% %--------------------------------------------------------------------------
%    nb_try=5;         % number of starting points   
%    [~,KmeansCent]                      =kmeans(Ckmeans,I,'replicates',nb_try);     
% %--------------------------------------------------------------------------
% %dT                       =size(features,2);
% 
% percent                  =2;
% referenceblock           =1:I*dT;
%   for t                  =1:I*dT
%       index              =(t-1)*I*dT+1: t*I*dT;
%      y(index)            =repmat(t,I*dT,1);
%   end  
%    z                      =repmat(referenceblock',I*dT,1);
% %--------------------------------------------------------------------------
% method                    =2; % 1. no scaling problem, 2, scaling calculation
%   [distance,~]            =distancecalculation(features.',method);
%    similarity            =distance;
%      x                   =reshape(similarity,(I*dT)^2,1); 
%    xx(:,1)               =y;
%    xx(:,2)               =z;
%    xx(:,3)               =x;
%    [gamma,rho,delta]    =originalcluster_dp(xx,percent);
%    [~,loc]               =sort(gamma,'descend'); 
%    Index=loc(1:I);
%    Disbetsampandcenter  =similarity(:,Index);
%    Cent                  =features(:,loc(1:I));
%    candidatefeatures  =features(:,loc(I+1:end));
% %--------------------------------------------------------------------------
% % [gamma]                 =decisiongraph(rho,delta,gamma,1,J,I);
% %  estH                    =Cent;
% % origH                   =KmeansCent.';
% % [~]                    =multidimensioncompress(estH,candidatefeatures,origH,1,I);
% %--------------------------------------------------------------------------
% for f=1:dT   
%          index     =(f-1)*I+1:f*I;              
%                  for np=1:size(nb_perm,1)               % for all permutations
%                        TestA      =Disbetsampandcenter(index,:);   
%                        cum =0;
%                     for i=1:I
%                        cost(np)  =cum+TestA(nb_perm(np,i),i);
%                     end
%                  end
%                  id=find(cost==min(cost));    % localize the best permutation
%                  permf=nb_perm(id(1),:);      % vector of permutation indexes
%                  Wf(:,:,f)=Wf(permf,:,f);     % demixing matrix after perm correction
%                  Hf(:,:,f)=Hf(:,permf,f);     % mixing matrix after perm correction
%                  Permutation(:,:,f)=permf;
% end    
% %-------------------------------------------------------------------------
% PP=KmeansCent.';
% 
%    for np=1:size(nb_perm,1) 
%       error(np)=   norm(Cent -PP(:,nb_perm(np,:)),'fro');
%    end
%        id=find(error==min(error));    % localize the best permutation
%      permf=nb_perm(id(1),:);      % vector of permutation indexes
% KmeansCent =PP(:,permf).';
% %--------------------------------------------------------------------------
%  for f=dT+1:sT   
%                  for np=1:size(nb_perm,1)               % for all permutations
%                      Ce_perm=Ce(:,nb_perm(np,:),f);     % permute the profile
%                      Kmeanscost(np) = norm(KmeansCent.' - Ce_perm,'fro');  % compute distance to centroid matrix
%                  end
%                  id=find(Kmeanscost==min(Kmeanscost));    % localize the best permutation
%                  permf=nb_perm(id(1),:);      % vector of permutation indexes
%                  Wf(:,:,f)=Wf(permf,:,f);     % demixing matrix after perm correction
%                  Hf(:,:,f)=Hf(:,permf,f);     % mixing matrix after perm correction
%                  Permutation(:,:,f)=permf;
%  end  
%end
%--------------------------------------------------------------------------
end